#!/usr/bin/env python

import os
import math
import sys
import persalys
import openturns as ot
import openturns.testing as ott

ssh_hostname = ""

# testcase A: parse output with tokens
with open("input_template.txt", "w") as f:
    f.write("X0=@X0\n")
    f.write("X1=@X1\n")
    f.write("X2=@X2\n")
with open("external_program.py", "w") as f:
    f.write("import sys\n")
    f.write("exec(open(sys.argv[1]).read())\n")
    f.write("Y0=X0+X1+X2\n")
    f.write("Y1=X0+X1*X2\n")
    f.write('with open("output.txt", "w") as f:\n')
    f.write('    f.write("Y0 [unit] = %.17e\\n" % Y0)\n')
    f.write('    f.write("$Y1=%.17e\\n" % Y1)\n')

with open("outputRef.txt", "w") as f:
    f.write("Y0 [unit] = 123.456\n")
    f.write("$Y1=7.89\n")

input_file = persalys.CouplingInputFile("input_template.txt")
input_file.setConfiguredPath("input.txt")
input_file.setVariables(
    ["X0", "X1", "X2"], ["@X0", "@X1", "@X2"], ["{:10.6f}", "{:10.6f}", "{:10.6f}"]
)
resource_file = persalys.CouplingResourceFile("external_program.py")
output_file = persalys.CouplingOutputFile("output.txt")
output_file.setVariables(["Y0", "Y1"], ["Y0 [unit] = ", "$Y1="], [0, 0], [0, 0], [0, 0])
step = persalys.CouplingStep(
    sys.executable + " external_program.py input.txt",
    [input_file],
    [resource_file],
    [output_file],
)

model = persalys.CouplingPhysicalModel("A", [step])
model.setSSHHostname(ssh_hostname)
print(output_file.checkOutputFile("outputRef.txt"))

# single evaluation
x = [1.0, 2.0, 3.0]
f = model.getFunction()
y = f(x)
print(y)
ott.assert_almost_equal(y, [6.0, 7.0])

# sampling test
ot_function = model.getFunction()
Y = ot.CompositeRandomVector(ot_function, ot.RandomVector(ot.Normal(3)))
y = Y.getSample(50)
mean, stddev = y.computeMean(), y.computeStandardDeviation()
print(mean, stddev)
ott.assert_almost_equal(mean, [0, 0], 0.3, 0.3)
ott.assert_almost_equal(stddev, [math.sqrt(3.0), 1.415], 0.3, 0.3)

# post-processing
code = 'import random\ndef _exec2(Y0, Y1):\n    A=r\\"U\\U-N\\N-u\\u-x\\x\\"\n    z = random.random()\n'
code += "    Y2 = Y0+3*Y1\n    Y3 = Y2+3*Y1\n    return Y2, Y3\n"
step.setCode(code)
model = persalys.CouplingPhysicalModel("B", [step])
model.setSSHHostname(ssh_hostname)
f = model.getFunction()
y = f(x)
print(y)
ott.assert_almost_equal(y, [6.0, 7.0, 27.0, 48.0])

# mis-spelled function name
step.setCode(
    "def _exe-c2(Y0, Y1):\n    Y2 = Y0+3*Y1\n    Y3 = Y2+3*Y1\n    return Y2, Y3\n"
)
model = persalys.CouplingPhysicalModel("C", [step])
model.setSSHHostname(ssh_hostname)
f = model.getFunction()
try:
    f(x)
    print("FAIL")
except Exception:
    print("OK")

# mis-spelled variable names
step.setCode(
    "def _exec2(Y-0, Y1):\n    Y2 = Y0+3*Y1\n    Y3 = Y2+3*Y1\n    return Y2, Y3\n"
)
model = persalys.CouplingPhysicalModel("D", [step])
model.setSSHHostname(ssh_hostname)
f = model.getFunction()
try:
    f(x)
    print("FAIL")
except RuntimeError:
    print("OK")

try:
    output_file.setVariables(["Y-0", "Y1"], ["Y0=", "Y1="], [0, 0], [0, 0], [0, 0])
    print("FAIL")
except TypeError:
    print("OK")


# cleanup
os.remove("input_template.txt")
os.remove("external_program.py")
os.remove("outputRef.txt")


# testcase D: parse output with table
with open("input_template.txt", "w") as f:
    f.write("X0=@X0\n")
    f.write("X1=@X1\n")
    f.write("X2=@X2\n")
with open("external_program.py", "w") as f:
    f.write("import sys\n")
    f.write("exec(open(sys.argv[1]).read())\n")
    f.write("Y0=X0+X1+X2\n")
    f.write("Y1=X0+X1*X2\n")
    f.write('with open("output.txt", "w") as f:\n')
    f.write('    f.write("%.17e\\n" % Y0)\n')
    f.write('    f.write("%.17e\\n" % Y1)\n')


input_file = persalys.CouplingInputFile("input_template.txt")
input_file.setConfiguredPath("input.txt")
input_file.setVariables(["X0", "X1", "X2"], ["@X0", "@X1", "@X2"], ["", "", ""])
resource_file = persalys.CouplingResourceFile("external_program.py")
output_file = persalys.CouplingOutputFile("output.txt")
output_file.setVariables(["Y0", "Y1"], [""] * 2, [0, 0], [0, -1], [0, 0])
step = persalys.CouplingStep(
    sys.executable + " external_program.py input.txt",
    [input_file],
    [resource_file],
    [output_file],
)
model = persalys.CouplingPhysicalModel("E", [step])
model.setSSHHostname(ssh_hostname)

# single evaluation
x = [1.0, 2.0, 3.0]
f = model.getFunction()
y = f(x)
print(y)
ott.assert_almost_equal(y, [6.0, 7.0])

# cleanup
os.remove("input_template.txt")
os.remove("external_program.py")

# testcase E: parse output and skip token
with open("input_template.txt", "w") as f:
    f.write("X0=@X0\n")
    f.write("X1=@X1\n")
    f.write("X2=@X2\n")
with open("external_program.py", "w") as f:
    f.write("import sys\n")
    f.write("exec(open(sys.argv[1]).read())\n")
    f.write("Y0=X0+X1+X2\n")
    f.write("Y1=X0+X1*X2\n")
    f.write('with open("output.txt", "w") as f:\n')
    # Voluntarily adding an empty line w. token so we can skip it
    f.write('    f.write("Y0=")\n')
    f.write('    f.write("Y0=%.3e\\n" % Y0)\n')
    f.write('    f.write("Y1=%.3e\\n" % Y1)\n')

input_file = persalys.CouplingInputFile("input_template.txt")
input_file.setConfiguredPath("input.txt")
input_file.setVariables(["X0", "X1", "X2"], ["@X0", "@X1", "@X2"], ["", "", ""])
resource_file = persalys.CouplingResourceFile("external_program.py")
output_file = persalys.CouplingOutputFile("output.txt")
output_file.setVariables(["Y0", "Y1"], ["Y0=", "Y1="], [1, 0], [0, 0], [0, 0])
step = persalys.CouplingStep(
    sys.executable + " external_program.py input.txt",
    [input_file],
    [resource_file],
    [output_file],
)
model = persalys.CouplingPhysicalModel("F", [step])
model.setSSHHostname(ssh_hostname)

# single evaluation
x = [1.0, 2.0, 3.0]
model.setCleanupWorkDirectory(False)
f = model.getFunction()
y = f(x)
print(y)
ott.assert_almost_equal(y, [6.0, 7.0])

# cleanup
os.remove("input_template.txt")
os.remove("external_program.py")


# testcase reusing temp dir
with open("input.txt.in", "w") as f:
    f.write("X0=@X0\n")
    f.write("X1=@X1\n")
    f.write("X2=@X2\n")
with open("program.py", "w") as f:
    f.write("import sys\n")
    f.write("exec(open(sys.argv[1]).read())\n")
    f.write("Y0=X0+X1+X2\n")
    f.write("Y1=X0+X1*X2\n")
    f.write('with open("output.txt", "w") as f:\n')
    f.write('    f.write("Y0=%.17e\\n" % Y0)\n')
    f.write('    f.write("Y1=%.17e\\n" % Y1)\n')


input_file = persalys.CouplingInputFile("input.txt.in")
input_file.setConfiguredPath("input.txt")
input_file.setVariables(["X0", "X1", "X2"], ["@X0", "@X1", "@X2"], ["", "", ""])
resource_file = persalys.CouplingResourceFile("program.py")
output_file = persalys.CouplingOutputFile("output.txt")
output_file.setVariables(["Y0", "Y1"], ["Y0=", "Y1="], [0, 0], [0, 0], [0, 0])
step = persalys.CouplingStep(
    sys.executable + " program.py input.txt",
    [input_file],
    [resource_file],
    [output_file],
)
model = persalys.CouplingPhysicalModel("reuse1", [step])
model.setSSHHostname(ssh_hostname)
# leave work dir
model.setCleanupWorkDirectory(False)

# single evaluation
x = [1.0, 2.0, 3.0]
f = model.getFunction()
y = f(x)
print(y)
ott.assert_almost_equal(y, [6.0, 7.0])

# reuse previous work dir, with an empty command
step = persalys.CouplingStep("", [input_file], [resource_file], [output_file])
model = persalys.CouplingPhysicalModel("reuse2", [step])
model.setSSHHostname(ssh_hostname)
f = model.getFunction()
y = f(x)
ott.assert_almost_equal(y, [6.0, 7.0])

# cleanup
os.remove("input.txt.in")
os.remove("program.py")

# testcase using file cache
with open("input.txt.in", "w") as f:
    f.write("X0=@X0\n")
    f.write("X1=@X1\n")
    f.write("X2=@X2\n")
with open("program.py", "w") as f:
    f.write("import sys\n")
    f.write("exec(open(sys.argv[1]).read())\n")
    f.write("Y0=X0+X1+X2\n")
    f.write("Y1=X0+X1*X2\n")
    f.write('with open("output.txt", "w") as f:\n')
    f.write('    f.write("Y0=%.17e\\n" % Y0)\n')
    f.write('    f.write("Y1=%.17e\\n" % Y1)\n')


input_file = persalys.CouplingInputFile("input.txt.in")
input_file.setConfiguredPath("input.txt")
input_file.setVariables(["X0", "X1", "X2"], ["@X0", "@X1", "@X2"], ["", "", ""])
resource_file = persalys.CouplingResourceFile("program.py")
output_file = persalys.CouplingOutputFile("output.txt")
output_file.setVariables(["Y0", "Y1"], ["Y0=", "Y1="], [0, 0], [0, 0], [0, 0])
step = persalys.CouplingStep(
    sys.executable + " program.py input.txt",
    [input_file],
    [resource_file],
    [output_file],
)
model = persalys.CouplingPhysicalModel("csvcache", [step])
model.setSSHHostname(ssh_hostname)
model.setCacheFiles("in.csv", "out.csv")

# single evaluation
x = [1.0, 2.0, 3.0]
f = persalys.PhysicalModel(model).getFunction()
for i in range(5):
    y = f(x)
print(y)
print("n calls=", f.getEvaluationCallsNumber())

ott.assert_almost_equal(y, [6.0, 7.0])

# cleanup
os.remove("input.txt.in")
os.remove("program.py")
os.remove("in.csv")
os.remove("out.csv")

# testcase with several steps

# step1: Y0,Y1=f1(X0, X1, X2)
with open("input1.txt.in", "w") as f:
    f.write("X0=@X0\n")
    f.write("X1=@X1\n")
    f.write("X2=@X2\n")
with open("program1.py", "w") as f:
    f.write('exec(open("input1.txt").read())\n')
    f.write("Y0=X0+X1+X2\n")
    f.write("Y1=X0+X1*X2\n")
    f.write('with open("output1.txt", "w") as f:\n')
    f.write('    f.write("Y0=%.17e\\n" % Y0)\n')
    f.write('    f.write("Y1=%.17e\\n" % Y1)\n')
input_file = persalys.CouplingInputFile("input1.txt.in")
input_file.setConfiguredPath("input1.txt")
input_file.setVariables(["X0", "X1", "X2"], ["@X0", "@X1", "@X2"], ["", "", ""])
resource_file = persalys.CouplingResourceFile("program1.py")
output_file = persalys.CouplingOutputFile("output1.txt")
output_file.setVariables(["Y0", "Y1"], ["Y0=", "Y1="], [0, 0], [0, 0], [0, 0])
step1 = persalys.CouplingStep(
    sys.executable + " program1.py", [input_file], [resource_file], [output_file]
)

# step2: Y2=f2(X2, X3, X4)
with open("input2.txt.in", "w") as f:
    f.write("X2=@X2\n")
    f.write("X3=@X3\n")
    f.write("X4=@X4\n")
with open("program2.py", "w") as f:
    f.write('exec(open("input2.txt").read())\n')
    f.write("Y2=27+X2+X3+X4\n")
    f.write('with open("output2.txt", "w") as f:\n')
    f.write('    f.write("Y2=%.17e\\n" % Y2)\n')
input_file = persalys.CouplingInputFile("input2.txt.in")
input_file.setConfiguredPath("input2.txt")
input_file.setVariables(["X2", "X3", "X4"], ["@X2", "@X3", "@X4"], ["", "", ""])
resource_file = persalys.CouplingResourceFile("program2.py")
output_file = persalys.CouplingOutputFile("output2.txt")
output_file.setVariables(["Y2"], ["Y2="], [0], [0], [0])
step2 = persalys.CouplingStep(
    sys.executable + " program2.py", [input_file], [resource_file], [output_file]
)

# step3: Z0=f3(Y1, Y2)
with open("input3.txt.in", "w") as f:
    f.write("Y1=@Y1\n")
    f.write("Y2=@Y2\n")
with open("program3.py", "w") as f:
    f.write('exec(open("input3.txt").read())\n')
    f.write("Z0=100+Y1+Y2\n")
    f.write('with open("output3.txt", "w") as f:\n')
    f.write('    f.write("Z0=%.17e\\n" % Z0)\n')
input_file = persalys.CouplingInputFile("input3.txt.in")
input_file.setConfiguredPath("input3.txt")
input_file.setVariables(["Y1", "Y2"], ["@Y1", "@Y2"], ["", ""])
resource_file = persalys.CouplingResourceFile("program3.py")
output_file = persalys.CouplingOutputFile("output3.txt")
output_file.setVariables(["Z0"], ["Z0="], [0], [0], [0])
step3 = persalys.CouplingStep(
    sys.executable + " program3.py", [input_file], [resource_file], [output_file]
)

# step4: no inputs, no outputs
step4 = persalys.CouplingStep("echo 42", [], [], [])
step4.setIsShell(True)

model = persalys.CouplingPhysicalModel("multi", [step1, step2, step3, step4])
model.setSSHHostname(ssh_hostname)
print("input variables=", model.getInputNames())
print("output variables=", model.getOutputNames())

# single evaluation
x = [1.0, 2.0, 3.0, 4.0, 5.0]
f = model.getFunction()
y = f(x)
print(y)
ott.assert_almost_equal(y, [6.0, 7.0, 39.0, 146.0])

# test environment override
with open("program4.py", "w") as f:
    f.write("import persalys as prs\n")

resource_file = persalys.CouplingResourceFile("program4.py")
step5 = persalys.CouplingStep(sys.executable + " program4.py", [], [resource_file], [])
step5.setEnvironment(["PYTHONPATH", "PYTHONHOME"], ["", ""])
model2 = persalys.CouplingPhysicalModel("envTest", [step1, step5])
x = [1.0, 2.0, 3.0]
f = model2.getFunction()
try:
    y = f(x)
    # oddly conda still runs without PYTHONPATH/PYTHONHOME
    if "conda" in sys.version:
        raise ValueError("oh wow")
    print("FAIL")
except Exception:
    print("OK")

# cleanup
os.remove("input1.txt.in")
os.remove("program1.py")
os.remove("input2.txt.in")
os.remove("program2.py")
os.remove("input3.txt.in")
os.remove("program3.py")
os.remove("program4.py")

# script
myStudy = persalys.Study("myStudy")
myStudy.add(model)
script = myStudy.getPythonScript()
# print('script=', script)
exec(script)



#! /usr/bin/env python

import persalys
import inspect

# find all instantiable classes
instantiables = []
for mod in [persalys]:
    for name, obj in inspect.getmembers(mod):
        if inspect.isclass(obj):
            cn = obj.__name__
            if "_" in cn:
                continue
            try:
                instance = obj()
                instantiables.append(obj)
            except Exception:
                pass

# find missing docstrings
count_class = 0
count_class_undoc = 0
count_methods = 0
count_methods_undoc = 0
for class_ in instantiables:
    count_class += 1
    if class_.__doc__ is None:
        print(f"{class_.__name__} class")
        count_class_undoc += 1
    for attr_name in dir(class_):
        if attr_name[0] == "_":
            continue
        class_attr = getattr(class_, attr_name)
        if callable(class_attr):
            count_methods += 1
            if class_attr.__doc__ is None:
                print(f"{class_.__name__}.{attr_name} method")
                count_methods_undoc += 1

# find all static functions
for name, mod in inspect.getmembers(persalys):
    if inspect.ismodule(mod):
        modn = mod.__name__
        if not modn.startswith("persalys") or modn == "persalys":
            continue
        if "_" in modn:
            continue

        for attr_name in dir(mod):
            if "_" in attr_name:
                continue
            obj = getattr(mod, attr_name)

            if not inspect.isclass(obj) and callable(obj):
                symboln = f"{modn.split('.')[1]}.{attr_name}"
                count_methods += 1
                if obj.__doc__ is None:
                    count_methods_undoc += 1
                    print(f"{symboln} method")


print(
    f"-- undocumented classes: {count_class_undoc} ({100.0 * count_class_undoc / count_class:.2f}%) --"
)
print(
    f"-- undocumented methods: {count_methods_undoc} ({100.0 * count_methods_undoc / count_methods:.2f}%) --"
)
if count_class_undoc + count_methods_undoc > 150:
    raise ValueError(
        f"too much undocumented class/methods ({count_class_undoc + count_methods_undoc})"
    )



import persalys
import openturns.testing as ott
import openturns as ot
from math import nan
from sys import stderr

study = persalys.Study("DataModel_nan")

# Model 1
model_1 = persalys.DataModel("Model_1", "jdd_test_avec_NaN.txt", [0, 1], [2])
study.add(model_1)

analysis_1 = persalys.DataAnalysis("analysis_1", model_1)
study.add(analysis_1)
analysis_1.run()

result_1 = analysis_1.getResult()
ott.assert_almost_equal(result_1.getMean()[0][0], 209712)
ott.assert_almost_equal(result_1.getMean()[2][0], 161441)
ott.assert_almost_equal(result_1.getEffectiveSize(), ot.Point([20, 9, 26]))

# Model 2
model_2 = persalys.DataModel("Model_2", "jdd_test_no_multivar.txt", [0, 1], [2])
study.add(model_2)

analysis_2 = persalys.DataAnalysis("analysis_2", model_2)
study.add(analysis_2)
analysis_2.run()

result_2 = analysis_2.getResult()
ott.assert_almost_equal(result_2.getMultivariateDoE().getSample().getSize(), 0)

# Model 3
model_3 = persalys.DataModel("Model_3", "jdd_test_empty_marginal.txt", [0, 1], [2])
study.add(model_3)

analysis_3 = persalys.DataAnalysis("analysis_3", model_3)
study.add(analysis_3)
analysis_3.run()

result_3 = analysis_3.getResult()
ott.assert_almost_equal(result_3.getDesignOfExperiment().getSample().getDimension(), 2)

# Model 3 modification
model_3.setInputSample(ot.Sample([[0, 1], [1, 0], [1, nan]]))
model_3.setOutputSample(ot.Sample([[1], [1], [nan]]))

analysis_3_2 = persalys.DataAnalysis("analysis_3_2", model_3)
study.add(analysis_3_2)
analysis_3_2.run()

result_3_2 = analysis_3_2.getResult()
ott.assert_almost_equal(result_3_2.getEffectiveSize(), ot.Point([3, 2, 2]))

# Model 4
print("model 4", file=stderr)
model_4 = persalys.DataModel("Model_4", "jdd_test_avec_NaN.txt", [0, 1, 2])
study.add(model_4)

analysis_4 = persalys.DataAnalysis("analysis_4", model_4)
study.add(analysis_4)
analysis_4.run()



#! /usr/bin/env python

import openturns as ot
import persalys

# from openturns.viewer import View

myStudy = persalys.Study("myStudy")

# Model
X0 = persalys.Input("X0", 0)
X2 = persalys.Input("X2", 0)
X1 = persalys.Input("X1", 0)

Y0 = persalys.Output("Y0")
Y2 = persalys.Output("Y2")
Y1 = persalys.Output("Y1")

bounds = ot.Interval([0.0, 0.0, 0.0], [5.0, 0.0, 5.0])
model = persalys.SymbolicPhysicalModel(
    "aModelPhys",
    [X0, X2, X1],
    [Y0, Y2, Y1],
    ["X0", "X1", "var g := 1.0 + 9.0 * (X0 + X1); g * (1.0 - sqrt(X0 / g))"],
)
myStudy.add(model)

# Optim #
ngen = 10
analysis = persalys.MultiObjectiveOptimizationAnalysis("optim", model, "nsga2")
analysis.setInterestVariables(["Y0", "Y1"])
analysis.addConstraint("Y0<3.")
analysis.setVariableInputs(["X0", "X1"])
analysis.setBounds(bounds)
analysis.setPopulationSize(50)
analysis.setGenerationNumber(ngen)
types = [
    ot.OptimizationProblemImplementation.CONTINUOUS,
    ot.OptimizationProblemImplementation.CONTINUOUS,
    ot.OptimizationProblemImplementation.INTEGER,
]
analysis.setVariablesType(types)
myStudy.add(analysis)
analysis.run()
print("analysis=", analysis)
results = analysis.getResult()

print("results=\n", results)
print("final population=\n", results.getFinalPop())

# script
script = myStudy.getPythonScript()
print(script)
exec(script)



#!/usr/bin/env python

import openturns as ot
import openturns.testing
import persalys

ot.RandomGenerator.SetSeed(0)

myStudy = persalys.Study("myStudy")
persalys.Study.Add(myStudy)

# Models
x1 = persalys.Input("x1", 0.2)
x2 = persalys.Input("x2", 1.2)
x3 = persalys.Input("x3", 1.0)
fake_var = persalys.Output("fake_var")
fake_var.setIsSelected(False)
y0 = persalys.Output("y0")
fake_y0 = persalys.Output("fake_y0")
y1 = persalys.Output("y1")

# symboilc model ##
formula_fake_var = "x1"
formula_y0 = "cos(0.5*x1) + sin(x2)"
formula_y1 = "cos(0.5*x1) + sin(x2) + x3"
symbolicModel = persalys.SymbolicPhysicalModel(
    "symbolicModel",
    [x1, x2, x3],
    [fake_var, y0, fake_y0, y1],
    [formula_fake_var, formula_y0, formula_y0, formula_y1],
)

myStudy.add(symbolicModel)

# python model ##
code = "from math import cos, sin, sqrt\n\ndef _exec(x1, x2, x3):\n    y0 = cos(0.5*x1) + sin(x2) + sqrt(x3)\n    return y0\n"
pythonModel = persalys.PythonPhysicalModel("pythonModel", [x1, x2, x3], [y0], code)
myStudy.add(pythonModel)

# coupling model ##
E = persalys.Input("E", 0, "")
F = persalys.Input("F", 0, "")
II = persalys.Input("I", 0, "")
L = persalys.Input("L", 0, "")
deviation = persalys.Output("deviation", "")
deviation.setValue(1.26262626263)
inputs = [E, F, II, L]
outputs = [deviation]
steps = []
input_files = []
input_file0 = persalys.CouplingInputFile(
    "../python/test/coupling_model/beam_input_template.xml"
)
input_file0.setConfiguredPath("beam_input.xml")
input_file0.setVariables(
    ["E", "F", "I", "L"], ["@E", "@F", "@I", "@L"], ["{}", "{}", "{}", "{}"]
)
input_files.append(input_file0)
resource_files = []
resource_file0 = persalys.CouplingResourceFile("../python/test/coupling_model/beam.py")
resource_files.append(resource_file0)
output_files = []
output_file0 = persalys.CouplingOutputFile("beam_output.txt")
output_file0.setVariables(["deviation"], ["deviation="], [0], [0], [0])
output_files.append(output_file0)
step0 = persalys.CouplingStep(
    "python3 beam.py", input_files, resource_files, output_files
)
step0.setIsShell(False)
steps.append(step0)
couplingModel = persalys.CouplingPhysicalModel("couplingModel", inputs, outputs, steps)
couplingModel.setCleanupWorkDirectory(True)
couplingModel.setCacheFiles("", "")
myStudy.add(couplingModel)
filename = "data_da.csv"
cDist = ot.JointDistribution(
    [ot.Normal(), ot.Gumbel(), ot.Normal(), ot.Uniform()],
    ot.BlockIndependentCopula([ot.IndependentCopula(2), ot.GumbelCopula()]),
)
sample = cDist.getSample(200)
sample.add([float("nan"), float("inf"), 0.0, 0.0])
sample.exportToCSVFile(filename, " ")

# Designs of Experiment ##

# fixed design ##
ot.RandomGenerator.SetSeed(0)
fixedDesign = persalys.FixedDesignOfExperiment("fixedDesign", symbolicModel)
inputSample = ot.LHSExperiment(
    ot.JointDistribution([ot.Uniform(0.0, 10.0), ot.Uniform(0.0, 10.0)]), 10
).generate()
inputSample.stack(ot.Sample(10, [0.5]))
fixedDesign.setOriginalInputSample(inputSample)
fixedDesign.run()
myStudy.add(fixedDesign)

# grid ##
values = [[0.5 + i * 1.5 for i in range(7)], [0.5 + i * 1.5 for i in range(7)], [1]]
grid = persalys.GridDesignOfExperiment("grid", symbolicModel, values)
myStudy.add(grid)

# importDesign ##
importDesign = persalys.ImportedDesignOfExperiment(
    "importDesign", symbolicModel, filename, [0, 2, 3]
)
importDesign.run()
myStudy.add(importDesign)

# onePointDesign ##
onePointDesign = persalys.GridDesignOfExperiment("onePointDesign", pythonModel)
myStudy.add(onePointDesign)

# twoPointsDesign ##
twoPointsDesign = persalys.GridDesignOfExperiment(
    "twoPointsDesign", pythonModel, [[0.2], [1.2], [-0.2, 1.0]]
)
myStudy.add(twoPointsDesign)

# fixed DataModel ##
fixedDataModel = persalys.DataModel(
    "fixedDataModel",
    fixedDesign.getOriginalInputSample(),
    fixedDesign.getResult().getDesignOfExperiment().getOutputSample(),
)
myStudy.add(fixedDataModel)

# imported DataModel ##
importDataModel = persalys.DataModel(
    "importDataModel", filename, [0, 2, 3], [1], ["x_0", "x_2", "x_3"], ["x_1"]
)
myStudy.add(importDataModel)

# model evaluation
evaluation1 = persalys.ModelEvaluation("evaluation1", symbolicModel)
myStudy.add(evaluation1)

# morris ##
try:
    morris = persalys.MorrisAnalysis("aMorris", symbolicModel)
    morris.setInterestVariables(["y0"])
    morris.setLevel(4)
    morris.setTrajectoriesNumber(10)
    morris.setSeed(2)
    myStudy.add(morris)
except Exception:
    print("No Morris")

# optimization ##
optim = persalys.OptimizationAnalysis("optim", symbolicModel, "TNC")
optim.setInterestVariables(["y1"])
optim.setVariableInputs(["x1", "x2"])
optim.setMaximumCallsNumber(150)
optim.setMaximumAbsoluteError(1e-6)
optim.setMaximumRelativeError(1e-6)
optim.setMaximumResidualError(1e-6)
optim.setMaximumConstraintError(1e-6)
myStudy.add(optim)

# multi-objective optimization
if ot.PlatformInfo.HasFeature("pagmo"):
    mooptim = persalys.MultiObjectiveOptimizationAnalysis(
        "mooptim", symbolicModel, "nsga2"
    )
    mooptim.setInterestVariables(["y0", "y1"])
    mooptim.addConstraint("y0    > 2")
    mooptim.setPopulationSize(60)
    mooptim.setGenerationNumber(12)
    mooptim.setVariableInputs(["x1", "x2", "x3"])
    myStudy.add(mooptim)

# calibration ##

# a- observations
nbObs = 100
inObs = ot.Uniform(0.0, 10.0).getSample(nbObs)
inObs.setDescription(["x1"])

y0Noise = ot.Normal(0, 0.1).getSample(nbObs)
y0Sample = ot.ParametricFunction(symbolicModel.getFunction("y0"), [1, 2], [1.2, 1.0])(
    inObs
)

y0Obs = y0Sample + y0Noise
y0Obs.setDescription(["y0"])
obs = persalys.Observations("observations", symbolicModel, inObs, y0Obs)
myStudy.add(obs)

# b- calibrationAnalysis
calibration = persalys.CalibrationAnalysis("calibration", obs)
calibration.setCalibratedInputs(["x2"], ot.Dirac([1.2]), ["x3"], [1.1])
calibration.setMethodName("GaussianNonlinear")
sigma = 0.15
errorCovariance = ot.CovarianceMatrix(1)
errorCovariance[0, 0] = sigma**2
calibration.setErrorCovariance(errorCovariance)
calibration.setBootStrapSize(25)
calibration.setConfidenceIntervalLength(0.99)

optimAlgo = calibration.getOptimizationAlgorithm()
optimAlgo.setMaximumCallsNumber(50)
optimAlgo.setMaximumAbsoluteError(1e-6)
optimAlgo.setMaximumRelativeError(1e-6)
optimAlgo.setMaximumResidualError(1e-6)
optimAlgo.setMaximumConstraintError(1e-6)
calibration.setOptimizationAlgorithm(optimAlgo)

myStudy.add(calibration)



import openturns as ot
import openturns.testing as ott
import persalys
from math import sin, exp

myStudy = persalys.Study("myStudy")

# +++++++++++++++++++++++++++++++++ Model 1 +++++++++++++++++++++++++++++++++ #
# simple example with a data model
model1 = persalys.DataModel(
    "model1", "Housing-prices-Boston.csv", list(range(13)), [13]
)
myStudy.add(model1)
print(model1)
print("inputNames=", model1.getInputNames())
print("outputNames=", model1.getOutputNames())

# Sensitivity analysis
analysis1 = persalys.DataSensitivityAnalysis("analysis1", model1)
myStudy.add(analysis1)
analysis1.run()

result1 = analysis1.getResult()
ott.assert_almost_equal(
    result1.getFirstOrderSobolIndices()[0],
    [
        0.269789,
        0.712661,
        0.709397,
        0.687175,
        0.698848,
        0.626429,
        0.166526,
        0.240896,
        0.69844,
        0.71514,
        0.707562,
        0.267871,
        0.649162,
    ],
)
ott.assert_almost_equal(
    result1.getFirstOrderSobolIndicesInterval()[0].getLowerBound(),
    [
        0.190449,
        0.0968657,
        0.325595,
        -0.104384,
        0.497543,
        0.541836,
        0.0273068,
        0.123202,
        0.0962543,
        0.243149,
        0.268534,
        0.0559757,
        0.600002,
    ],
)
ott.assert_almost_equal(
    result1.getFirstOrderSobolIndicesInterval()[0].getUpperBound(),
    [
        0.363531,
        0.307685,
        0.547926,
        0.142314,
        0.647108,
        0.688973,
        0.218533,
        0.33305,
        0.314734,
        0.460371,
        0.482617,
        0.225703,
        0.699414,
    ],
)

ott.assert_almost_equal(
    result1.getSignedSRCIndices(),
    [
        [
            -0.100175,
            0.117651,
            0.0155602,
            0.0742493,
            -0.224215,
            0.290666,
            0.00229872,
            -0.33788,
            0.289376,
            -0.225936,
            -0.22444,
            0.0932346,
            -0.407997,
        ]
    ],
)
ott.assert_almost_equal(
    result1.getSRCIndices(), [[x**2 for x in result1.getSignedSRCIndices()[0]]]
)

ott.assert_almost_equal(
    result1.getSignedSRCIndicesInterval()[0].getLowerBound()[:3],
    [-0.151228, 0.0624121, -0.0406055],
)
ott.assert_almost_equal(
    result1.getSignedSRCIndicesInterval()[0].getUpperBound()[:3],
    [-0.0307025, 0.171362, 0.0924589],
)

ott.assert_almost_equal(result1.getR2()[0], 0.638872)

# +++++++++++++++++++++++++++++++++ Model 2 +++++++++++++++++++++++++++++++++ #
# Model with several outputs and dependences between inputs

# ------------------------------model definition----------------------------- #
marginals = [
    ot.Normal(),  # X0
    ot.Uniform(0, 1),  # X1
    ot.Normal(),  # X2
    ot.Normal(),  # X3
    ot.Normal(),  # X4
]
R = ot.CorrelationMatrix(5)
R[0, 2] = 0.3
R[3, 4] = -0.2
copula = ot.NormalCopula(R)
distribution = ot.JointDistribution(marginals, copula)
distribution.setDescription(["X0", "X1", "X2", "X3", "X4"])


def model2_def(X):
    x0, x1, x2, x3, x4 = X
    y0 = x0 + x1**2 + sin(x2)
    y1 = x3 * x4 + exp(-x0)
    y2 = x1 + x2 + x3 + x4
    return [y0, y1, y2]


ot_model2 = ot.PythonFunction(5, 3, model2_def)

sample_size = 300
ot.RandomGenerator.SetSeed(0)
sample = distribution.getSample(sample_size)
output_sample = ot_model2(sample)
output_sample.setDescription(["Y0", "Y1", "Y2"])
sample.stack(output_sample)


# saving the sample to a CSV file
filename = "DataSensitivityAnalysis_std.csv"
sample.exportToCSVFile(filename)

# ------------------------test of the persalys model------------------------- #
model2 = persalys.DataModel("model2", filename, list(range(5)), list(range(5, 8)))
myStudy.add(model2)
print()
print(model2)

analysis2 = persalys.DataSensitivityAnalysis("analysis2", model2)
myStudy.add(analysis2)
analysis2.run()

result2 = analysis2.getResult()
for i in range(3):
    print(f"Output Y{i} first order indices: ", result2.getFirstOrderSobolIndices()[i])
    print(
        f"Output Y{i} first order indices intervals: ",
        result2.getFirstOrderSobolIndicesInterval()[i],
    )

print("Is independent ?", result2.isIndependent())
print("Warning message: ", result2.getIndependenceWarningMessage())

# -------------------------test without output sample------------------------ #
model2_no_output = persalys.DataModel("model2_no_output", filename, list(range(5)), [])
myStudy.add(model2_no_output)

analysis2_no_output = persalys.DataSensitivityAnalysis(
    "analysis2_no_output", model2_no_output
)
myStudy.add(analysis2_no_output)
with ott.assert_raises(RuntimeError):
    analysis2_no_output.run()



#!/usr/bin/env python

import openturns as ot
import persalys

Study_0 = persalys.Study("Study_0")
persalys.Study.Add(Study_0)

# variables
z0 = persalys.Input("z0", 100, "")
v0 = persalys.Input("v0", 55, "")
m = persalys.Input("m", 80, "")
c = persalys.Input("c", 15, "")
z = persalys.Output("z", "")
inputs = [z0, v0, m, c]
outputs = [z]

# mesh model
meshModel = persalys.GridMeshModel(ot.Interval(0.0, 12.0), [20])

# Python model
code = "from math import exp\n\n"
code += "def _exec(z0,v0,m,c):\n    g = 9.81\n    zmin = 0.\n    tau = m / c\n    vinf = -m * g / c\n\n"
code += "    # mesh nodes\n    t = getMesh().getVertices()\n\n"
code += "    z = [max(z0 + vinf * t_i[0] + tau * (v0 - vinf) * (1 - exp(-t_i[0] / tau)), zmin) for t_i in t]\n"
code += "    return z"
model = persalys.PythonFieldModel("model", meshModel, inputs, outputs, code)
Study_0.add(model)

f = model.getPointToFieldFunction()
print(f([100, 55, 81, 17]))
print(f([[100, 55, 81, 17], [100, 55, 81, 18]]))

# model with an error : ZeroDivisionError
code = "from math import exp\n\n"
code += "def _exec(z0,v0,m,c):\n    g = 9.81 / 0.\n    zmin = 0.\n    tau = m / c\n    vinf = -m * g / c\n\n"
code += "    # mesh nodes\n    t = mesh.getVertices()\n\n"
code += "    z = [max(z0 + vinf * t_i[0] + tau * (v0 - vinf) * (1 - exp(-t_i[0] / tau)), zmin) for t_i in t]\n"
code += "    return z"
model.setCode(code)
model.setParallel(True)
f = model.getPointToFieldFunction()
try:
    print(f([[100, 55, 81, 17]]))
except Exception as e:
    ok = "ZeroDivisionError" in str(e)
    print(f"ZeroDivisionError occurred: {ok}")
    print("Error on the line 4 : %s" % ("line 4" in str(e)))
try:
    print(f([100, 55, 81, 17]))
except Exception as e:
    ok = "ZeroDivisionError" in str(e)
    print(f"ZeroDivisionError occurred: {ok}")
    print("Error on the line 4 : %s" % ("line 4" in str(e)))

# model with an error : wrong output type
model.setCode("def _exec(z0,v0,m,c):\n    z = z0\n    return z")

f = model.getPointToFieldFunction()

try:
    print(f([100, 55, 81, 17]))
except Exception as e:
    print(
        "InvalidArgumentException occurred: %s" % ("InvalidArgumentException" in str(e))
    )
try:
    print(f([[100, 55, 81, 17]]))
except Exception as e:
    print(
        "InvalidArgumentException occurred: %s" % ("InvalidArgumentException" in str(e))
    )

# model with an error : wrong output sequence size
model.setCode("def _exec(z0,v0,m,c):\n    z = [z0]*2\n    return z")

f = model.getPointToFieldFunction()

try:
    print(f([100, 55, 81, 17]))
except Exception as e:
    print(
        "InvalidArgumentException occurred: %s" % ("InvalidArgumentException" in str(e))
    )
try:
    print(f([[100, 55, 81, 17]]))
except Exception as e:
    print(
        "InvalidArgumentException occurred: %s" % ("InvalidArgumentException" in str(e))
    )

# Python model with two outputs
z1 = persalys.Output("z1", "")
outputs = [z, z1]

code = "from math import exp\n\ndef _exec(z0,v0,m,c):\n    g = 9.81\n    zmin = 0.\n    tau = m / c\n    vinf = -m * g / c\n\n"
code += "    # mesh nodes\n    t = getMesh().getVertices()\n\n"
code += "    z = [max(z0 + vinf * t_i[0] + tau * (v0 - vinf) * (1 - exp(-t_i[0] / tau)), zmin) for t_i in t]\n    z1 = z[:]\n"
code += "    return z, z1"
model2 = persalys.PythonFieldModel("model2", meshModel, inputs, outputs, code)
model2.setParallel(True)
Study_0.add(model2)

f = model2.getPointToFieldFunction()

print(f([100, 55, 81, 17]))
print(f([[100, 55, 81, 17], [100, 55, 81, 18]]))


# model2 with an error : wrong output sequence size
code = "from math import exp\n\n"
code += "def _exec(z0,v0,m,c):\n    g = 9.81\n    zmin = 0.\n    tau = m / c\n    vinf = -m * g / c\n\n"
code += "    # mesh nodes\n    t = getMesh().getVertices()\n\n"
code += "    z = [max(z0 + vinf * t_i[0] + tau * (v0 - vinf) * (1 - exp(-t_i[0] / tau)), zmin) for t_i in t]\n    z1 = [2.]\n"
code += "    return z, z1"
model.setCode(code)

f = model2.getPointToFieldFunction()

try:
    print(f([100, 55, 81, 17]))
except Exception as e:
    print(
        "InvalidArgumentException occurred: %s" % ("InvalidArgumentException" in str(e))
    )
try:
    print(f([[100, 55, 81, 17]]))
except Exception as e:
    print(
        "InvalidArgumentException occurred: %s" % ("InvalidArgumentException" in str(e))
    )


# script
script = Study_0.getPythonScript()
print(script)
exec(script)



#!/usr/bin/env python

import openturns as ot
import persalys

Study_0 = persalys.Study("Study_0")
persalys.Study.Add(Study_0)

# variables
z0 = persalys.Input("z0", 100, "")
v0 = persalys.Input("v0", 55, "")
m = persalys.Input("m", 80, "")
c = persalys.Input("c", 15, "")
z = persalys.Output("z", "")
z2 = persalys.Output("z2", "")
inputs = [z0, v0, m, c]
outputs = [z]

# mesh model
meshModel = persalys.GridMeshModel(ot.Interval(0.0, 12.0), [20])

# Symbolic model
formula = [
    "max(0, z0 - (m * 9.81 / c) * t + (m / c) * (v0 + (m * 9.81 / c)) * (1 - exp(-t * c / m)))"
]
model1 = persalys.SymbolicFieldModel("model1", meshModel, inputs, outputs, formula)
Study_0.add(model1)

f = model1.getPointToFieldFunction()
print(f([100, 55, 81, 17]))
print(f([[100, 55, 81, 17], [100, 55, 81, 18]]))


# Symbolic model : two outputs
outputs = [z, z2]
formula = [
    "max(0, z0 - (m * 9.81 / c) * t + (m / c) * (v0 + (m * 9.81 / c)) * (1 - exp(-t * c / m)))"
] * 2
model2 = persalys.SymbolicFieldModel("model2", meshModel, inputs, outputs, formula)
Study_0.add(model2)

f = model2.getPointToFieldFunction()
print(f([100, 55, 81, 17]))
print(f([[100, 55, 81, 17], [100, 55, 81, 18]]))


# script
script = Study_0.getPythonScript()
print(script)
# exec(script)



#! /usr/bin/env python

import persalys

width = 40

# check for Morris method
print("1: Morris method (otmorris)".ljust(width), end=" ")
try:
    persalys.MorrisResult()
    print("ok")
except Exception:
    print("n/a")

# check for FMI support
print("2: FMI (otfmi)".ljust(width), end=" ")
try:
    persalys.FMUInfo()
    print("ok")
except Exception:
    print("n/a")

# check for YACS support
print("3: YACS (salome)".ljust(width), end=" ")
try:
    persalys.YACSPhysicalModel()
    print("ok")
except Exception:
    print("n/a")



#! /usr/bin/env python

import openturns as ot
import openturns.testing
import persalys
import os

myStudy = persalys.Study("myStudy")

# data
filename = "données.csv"
ot.RandomGenerator.SetSeed(0)
ot.Normal(3).getSample(10).exportToCSVFile(filename)
inColumns = [0, 2]

# Model 1
model = persalys.DataModel("myDataModel", filename, inColumns)
myStudy.add(model)
print(model)

# Model 2
model2 = persalys.SymbolicPhysicalModel(
    "SM", [persalys.Input("A"), persalys.Input("B")], [persalys.Output("S")], ["A+B+2"]
)
myStudy.add(model2)
importedDOE = persalys.ImportedDesignOfExperiment("doeI", model2, filename, inColumns)
myStudy.add(importedDOE)

# script
script = myStudy.getPythonScript()
print(script)

# save
xmlFileName = "file_with_données.xml"
myStudy.save(xmlFileName)

# open
s = persalys.Study.Open("file_with_données.xml")
print(s.getPythonScript())

os.remove(filename)
os.remove(xmlFileName)
os.remove(xmlFileName.split(".")[0] + ".h5")



"""
    Persalys --- An OpenTURNS module
    ==================================

    Contents
    --------
      'Persalys' is a module for OpenTURNS

"""

# flake8: noqa

# ensures swig type tables order & dll load
import openturns as _ot

from .persalys import *

__version__ = "19.1"



#! /usr/bin/env python

import openturns as ot
import persalys

myStudy = persalys.Study("myStudy")

# Model
X0 = persalys.Input("X0", ot.Normal(1, 1))
X1 = persalys.Input("X1", ot.Normal(1, 1))
Y0 = persalys.Output("Y0")

model = persalys.SymbolicPhysicalModel("aModelPhys", [X0, X1], [Y0], ["sin(X0) + 8*X1"])
myStudy.add(model)

# limit state ##
limitState = persalys.LimitState("aLimitState", model, "Y0", ot.Greater(), 20.0)
myStudy.add(limitState)

# IS ##
analysis = persalys.ImportanceSamplingAnalysis("myIS", limitState)
analysis.setMaximumCalls(1000)
analysis.setBlockSize(10)
analysis.setSeed(2)
myStudy.add(analysis)
print(analysis)

analysis.run()

print("result=", analysis.getResult())

# IS ##
analysis2 = persalys.ImportanceSamplingAnalysis("myIS2", limitState)
analysis2.setMaximumCoefficientOfVariation(0.02)
analysis2.setMaximumElapsedTime(100000)
analysis2.setBlockSize(100)
myStudy.add(analysis2)
print(analysis2)

analysis2.run()

print("result=", analysis2.getResult())

# IS ##
X2 = persalys.Input("X2", 2)
model.addInput(X2)
model.setFormula("Y0", "sin(X0) + 8*X1 + X2")

analysis3 = persalys.ImportanceSamplingAnalysis("myIS3", limitState)
analysis3.setMaximumCalls(1000)
analysis3.setBlockSize(10)
myStudy.add(analysis3)
print(analysis3)

analysis3.run()

result3 = analysis3.getResult()
print("result=", result3)

# IS ##
analysis4 = persalys.ImportanceSamplingAnalysis("myIS4", limitState)
analysis4.setMaximumCalls(1000)
analysis4.setBlockSize(10)
analysis4.setMaximumCoefficientOfVariation(-1.0)
analysis4.setStandardSpaceDesignPoint([0.0686039, 1.14043])

myStudy.add(analysis4)
print(analysis4)

analysis4.run()

result4 = analysis4.getResult()
print("result=", result4)

# script
script = myStudy.getPythonScript()
print(script)
exec(script)
